#!/bin/bash
NAMESPACE=$(getProdToNamespacesMapping aimanager $PROD_NAMESPACES_REFFILE_FINAL)
AIOPS_INSTALL_NAME=$($OC_CMD get install -n $NAMESPACE -o jsonpath='{.items[*].metadata.name}')
PG_CLUSTER=

### The Following Script is used to rebuild the postgres cluster in an AIOPS 4.6.1 Deploy ####
### The Rebuild of the postgres cluster will cause all data to be lost from the DB ###

check_postgres_status() {
	local CLUSTER_WAIT_CTR=0
	local CLUSTER_WAIT_TIMEOUT=30
	local CLUSTER_WAIT_SLEEP=60
    
	while true; do
		(( CLUSTER_WAIT_CTR+=1 ))

		if [ $CLUSTER_WAIT_CTR -gt $CLUSTER_WAIT_TIMEOUT ]
		then
			echo "[$(date)] [$(basename "$0")] [FATAL] Postgres cluster is unable to get into ready state! Returning..."
			echo "[$(date)] [$(basename "$0")] [FATAL] Postgres cluster is unable to get into ready state! Returning..." >&9
			return 1
		fi

		local message=$($OC_CMD get cluster "$PG_CLUSTER" -n $NAMESPACE -o jsonpath='{.status.conditions[0].message}')
		
		if [[ "$message" == "Cluster is Ready" ]]; then
			echo "[$(date)] [${FUNCNAME[0]}] Cluster is ready [STATUS = $message]"
			echo "[$(date)] [${FUNCNAME[0]}] Cluster is ready [STATUS = $message]" >&8

			break
		else
			echo "[$(date)] [${FUNCNAME[0]}] Waiting for cluster to be ready [STATUS = $message]..."
			echo "[$(date)] [${FUNCNAME[0]}] Waiting for cluster to be ready [STATUS = $message]..." >&8

			sleep $CLUSTER_WAIT_SLEEP
		fi
	done
}

drop_flyway_schema_history() {
	local POSTGRES_PODNAME=$(getResourceInstances pod $POSTGRES_KEEPER_PODNAME $NAMESPACE | head -1)
	local POSTGRES_SECRET="${AIOPS_INSTALL_NAME}-edb-postgres-superuser"

	local POSTGRES_SECRET_OUTFILE="$OUTDIR/$POSTGRES_SECRET.out"
	$OC_CMD get -o yaml -n $NAMESPACE secret $POSTGRES_SECRET > $POSTGRES_SECRET_OUTFILE

     local POSTGRES_HOSTNAME=$(grep -E '^[ ]*host: ' $POSTGRES_SECRET_OUTFILE | $AWK -F: '{ print $2 }' | sed 's/^[ ]*//' | base64 -d)
     local POSTGRES_PORT=$(grep -E '^[ ]*port: ' $POSTGRES_SECRET_OUTFILE | $AWK -F: '{ print $2 }' | sed 's/^[ ]*//' | base64 -d)
     local POSTGRES_USERNAME=$(grep -E '^[ ]*username: ' $POSTGRES_SECRET_OUTFILE | head -1 | $AWK -F: '{ print $2 }' | sed 's/^[ ]*//' | base64 -d)
     local POSTGRES_PASSWORD=$(grep -E '^[ ]*password: ' $POSTGRES_SECRET_OUTFILE | head -1 | $AWK -F: '{ print $2 }' | sed 's/^[ ]*//' | base64 -d)
	local POSTGRES_DBNAME="aiops_topology"

	local POSTGRES_CONN_STR="host=$POSTGRES_HOSTNAME port=$POSTGRES_PORT user=$POSTGRES_USERNAME password=$POSTGRES_PASSWORD dbname=$POSTGRES_DBNAME sslmode=prefer"
	
	if [ $MG_DEBUG == "true" ]
	then
		echo "[$(date)] [${FUNCNAME[0]}] POSTGRES_SECRET = [$POSTGRES_SECRET] / POSTGRES_SECRET_OUTFILE = [$POSTGRES_SECRET_OUTFILE]"
		echo "[$(date)] [${FUNCNAME[0]}] POSTGRES_SECRET = [$POSTGRES_SECRET] / POSTGRES_SECRET_OUTFILE = [$POSTGRES_SECRET_OUTFILE]" >&8
		echo "[$(date)] [${FUNCNAME[0]}] POSTGRES_HOSTNAME = [$POSTGRES_HOSTNAME] / POSTGRES_PORT = [$POSTGRES_PORT] / POSTGRES_USERNAME = [$POSTGRES_USERNAME] / POSTGRES_PASSWORD = [${#POSTGRES_PASSWORD} (length)] / POSTGRES_DBNAME = [$POSTGRES_DBNAME]"
		echo "[$(date)] [${FUNCNAME[0]}] POSTGRES_HOSTNAME = [$POSTGRES_HOSTNAME] / POSTGRES_PORT = [$POSTGRES_PORT] / POSTGRES_USERNAME = [$POSTGRES_USERNAME] / POSTGRES_PASSWORD = [${#POSTGRES_PASSWORD} (length)] / POSTGRES_DBNAME = [$POSTGRES_DBNAME]" >&8
	fi

	if [ ${#POSTGRES_HOSTNAME} -eq 0 ] || [ ${#POSTGRES_PORT} -eq 0 ] || [ ${#POSTGRES_USERNAME} -eq 0 ] || [ ${#POSTGRES_PASSWORD} -eq 0 ] || [ ${#POSTGRES_DBNAME} -eq 0 ]
	then
		echo "[$(date)] [${FUNCNAME[0]}] Unable to construct Postgres connection string! Returning..."
		echo "[$(date)] [${FUNCNAME[0]}] Unable to construct Postgres connection string! Returning..." >&9
		return 1
	fi

	local EDB_PRIMARY=$($OC_CMD get cluster -n $NAMESPACE | grep "${AIOPS_INSTALL_NAME}-edb-postgres" | $AWK '{print $NF}')

	if [ $MG_DEBUG == "true" ]
	then
		echo "[$(date)] [${FUNCNAME[0]}] EDB_PRIMARY = [$EDB_PRIMARY]"
	fi
	
	rm $POSTGRES_SECRET_OUTFILE

	echo "[$(date)] [${FUNCNAME[0]}] Scaling down deployment [aiops-topology-inventory] to 0..."
	echo "[$(date)] [${FUNCNAME[0]}] Scaling down deployment [aiops-topology-inventory] to 0..." >&8
	$OC_CMD scale deployment aiops-topology-inventory -n $NAMESPACE --replicas=0
	sleep 10

	# TODO: Check scale = 0
	
	if isPodOk $NAMESPACE $EDB_PRIMARY > /dev/null 2>&1
	then
		local POSTGRES_CMD="psql '$POSTGRES_CONN_STR' -c \\\"DROP TABLE \\\"flyway_schema_history\\\";\\\""
		local EXEC_CMD="$OC_CMD exec $EDB_PRIMARY -n $NAMESPACE -c postgres -- bash -c \"$POSTGRES_CMD\""

		if [ $MG_DEBUG == "true" ]
		then
			local POSTGRES_CMD_FOR_ECHO=$(echo "$POSTGRES_CMD" | sed 's/password=.* dbname=/password=XXX dbname=/g')
			local EXEC_CMD_FOR_ECHO=$(echo "$EXEC_CMD" | sed 's/password=.* dbname=/password=XXX dbname=/g')

			echo "[$(date)] [${FUNCNAME[0]}] POSTGRES_CMD = [$POSTGRES_CMD_FOR_ECHO]"
			echo "[$(date)] [${FUNCNAME[0]}] POSTGRES_CMD = [$POSTGRES_CMD_FOR_ECHO]" >&8
			echo "[$(date)] [${FUNCNAME[0]}] EXEC_CMD = [$EXEC_CMD_FOR_ECHO]"
			echo "[$(date)] [${FUNCNAME[0]}] EXEC_CMD = [$EXEC_CMD_FOR_ECHO]" >&8

			local CHECK_TABLE_CMD="$OC_CMD exec $EDB_PRIMARY -n $NAMESPACE -c postgres -- bash -c \"psql '$POSTGRES_CONN_STR' -t -c \\\"SELECT EXISTS (SELECT FROM information_schema.tables WHERE table_name = 'flyway_schema_history');\\\"\"" 
			local CHECK_TABLE_CMD_FOR_ECHO=$(echo "$CHECK_TABLE_CMD" | sed 's/password=.* dbname=/password=XXX dbname=/g')

			echo "[$(date)] [${FUNCNAME[0]}] [PRE-DELETE] CHECK_TABLE_CMD = [$CHECK_TABLE_CMD_FOR_ECHO]"
			echo "[$(date)] [${FUNCNAME[0]}] [PRE-DELETE] CHECK_TABLE_CMD = [$CHECK_TABLE_CMD_FOR_ECHO]" >&8

			local CHECK_TABLE_RES=$(eval "$CHECK_TABLE_CMD" | sed 's/^[[:space:]]*//')
			echo "[$(date)] [${FUNCNAME[0]}] [PRE_DELETE] CHECK_TABLE_RES = [$CHECK_TABLE_RES]"
			echo "[$(date)] [${FUNCNAME[0]}] [PRE_DELETE] CHECK_TABLE_RES = [$CHECK_TABLE_RES]" >&8
		fi

		echo "[$(date)] [${FUNCNAME[0]}] Dropping table [flyway_schema_history]..."
		echo "[$(date)] [${FUNCNAME[0]}] Dropping table [flyway_schema_history]..." >&8
		eval "$EXEC_CMD"

		if [ $MG_DEBUG == "true" ]
		then
			local CHECK_TABLE_CMD="$OC_CMD exec $EDB_PRIMARY -n $NAMESPACE -c postgres -- bash -c \"psql '$POSTGRES_CONN_STR' -t -c \\\"SELECT EXISTS (SELECT FROM information_schema.tables WHERE table_name = 'flyway_schema_history');\\\"\"" 
			local CHECK_TABLE_CMD_FOR_ECHO=$(echo "$CHECK_TABLE_CMD" | sed 's/password=.* dbname=/password=XXX dbname=/g')

			echo "[$(date)] [${FUNCNAME[0]}] [POST-DELETE] CHECK_TABLE_CMD = [$CHECK_TABLE_CMD_FOR_ECHO]"
			echo "[$(date)] [${FUNCNAME[0]}] [POST-DELETE] CHECK_TABLE_CMD = [$CHECK_TABLE_CMD_FOR_ECHO]" >&8

			local CHECK_TABLE_RES=$(eval "$CHECK_TABLE_CMD" | sed 's/^[[:space:]]*//')
			echo "[$(date)] [${FUNCNAME[0]}] [POST-DELETE] CHECK_TABLE_RES = [$CHECK_TABLE_RES]"
			echo "[$(date)] [${FUNCNAME[0]}] [POST-DELETE] CHECK_TABLE_RES = [$CHECK_TABLE_RES]" >&8
		fi
	else
		echo "[$(date)] [${FUNCNAME[0]}] Pod [EDB_PRIMARY] is not OK! Returning..."
		echo "[$(date)] [${FUNCNAME[0]}] Pod [EDB_PRIMARY] is not OK! Returning..." >&9
	fi
}

isClusterDeleted()
{
	local PG_CLUSTER_OLD_CREATION_TIME="$1"
	local PG_CLUSTER=$($OC_CMD get cluster -n $NAMESPACE | grep "${AIOPS_INSTALL_NAME}-edb-postgres" | $AWK '{print $1}')
	
	if [ ${#PG_CLUSTER} -eq 0 ]
	then
		# Cluster is deleted
		return 0
	else
		local PG_CLUSTER_NEW_CREATION_TIME=$($OC_CMD get cluster -n $NAMESPACE $PG_CLUSTER -o jsonpath='{.metadata.creationTimestamp}')
	
		if [ $MG_DEBUG == "true" ]
		then
			echo "[$(date)] [${FUNCNAME[0]}] PG_CLUSTER_OLD_CREATION_TIME = [$PG_CLUSTER_OLD_CREATION_TIME] / PG_CLUSTER_NEW_CREATION_TIME = [$PG_CLUSTER_NEW_CREATION_TIME]"
			echo "[$(date)] [${FUNCNAME[0]}] PG_CLUSTER_OLD_CREATION_TIME = [$PG_CLUSTER_OLD_CREATION_TIME] / PG_CLUSTER_NEW_CREATION_TIME = [$PG_CLUSTER_NEW_CREATION_TIME]" >&8
		fi

		if [ ${#PG_CLUSTER_NEW_CREATION_TIME} -eq 0 ]
		then
			echo "[$(date)] [${FUNCNAME[0]}] Unable to determine PG_CLUSTER_NEW_CREATION_TIME! Returning..."
			echo "[$(date)] [${FUNCNAME[0]}] Unable to determine PG_CLUSTER_NEW_CREATION_TIME! Returning..." >&9
	
			return 1
		fi
	
		if [ ${PG_CLUSTER_OLD_CREATION_TIME} -eq ${PG_CLUSTER_NEW_CREATION_TIME} ]
		then
			# Cluster is still there
			return 1
		else
			# Cluster is deleted
			return 0	
		fi
	fi
}

#### Main #####

echo -e "\e[31m**Caution: Running this script will result in data loss on the Postgres System**\e[0m"
echo -e "\e[31m**Caution: Running this script will result in data loss on the Postgres System**\e[0m" >&8

TO_ASK_DEL_POSTGRES="true"

while true; do
	if [ ${TO_ASK_DEL_POSTGRES} == "true" ]
	then
		if [ ${#AUTO_YES} -gt 0 ] && [ $AUTO_YES == "true" ]
		then
			echo "[$(date)] [$(basename "$0")] AUTO_YES = [$AUTO_YES] - Proceeding to delete Postgres Cluster..."
			echo "[$(date)] [$(basename "$0")] AUTO_YES = [$AUTO_YES] - Proceeding to delete Postgres Cluster..." >&8
			echo "Y" >&8
			DEL_POSTGRES="Y"
		else
			if [ ${#RTVAR_CMDEXEC_DEL_POSTGRES} -gt 0 ]
			then
				if [ ${RTVAR_CMDEXEC_DEL_POSTGRES} == "Y" ] || [ ${RTVAR_CMDEXEC_DEL_POSTGRES} == "y" ]
				then
					DEL_POSTGRES="$RTVAR_CMDEXEC_DEL_POSTGRES"
				else
					# Since 'timeout' command will be used to run the script, and it does not play well with 'read' command, we have to exit!	
					echo "[$(date)] [$(basename "$0")] [FATAL] User input CMDEXEC_DEL_POSTGRES [$RTVAR_CMDEXEC_DEL_POSTGRES] is invalid! Exiting..."
					echo "[$(date)] [$(basename "$0")] [FATAL] User input CMDEXEC_DEL_POSTGRES [$RTVAR_CMDEXEC_DEL_POSTGRES] is invalid! Exiting..." >&9
					exit 1
				fi
			else
				if [ ${#RTVAR_CMDEXEC_TIMEOUT} -gt 0 ] && [ ${RTVAR_CMDEXEC_TIMEOUT} == "false" ]
				then
					read -r -p "Do you want to delete the Postgres Cluster ? [Y/N] " DEL_POSTGRES <&7
				else
					echo "[$(date)] [$(basename "$0")] [FATAL] Since 'timeout' command will be used to run the script, and it does not play well with 'read' command, we have to exit!"
					echo "[$(date)] [$(basename "$0")] [FATAL] Since 'timeout' command will be used to run the script, and it does not play well with 'read' command, we have to exit!" >&9
					exit 1
				fi
			fi

			echo "$DEL_POSTGRES" >&8
		fi
	fi

	case $DEL_POSTGRES in
		[Yy])
			# code to execute if user wants to delete the Postgres Cluster
			PG_CLUSTER=$($OC_CMD get cluster -n $NAMESPACE | grep -i "${AIOPS_INSTALL_NAME}-edb-postgres" | $AWK '{print $1}')

			if [ $MG_DEBUG == "true" ]
			then
				echo "[$(date)] [$(basename "$0")] AIOPS_INSTALL_NAME = [$AIOPS_INSTALL_NAME]"
				echo "[$(date)] [$(basename "$0")] AIOPS_INSTALL_NAME = [$AIOPS_INSTALL_NAME]" >&8
				echo "[$(date)] [$(basename "$0")] PG_CLUSTER = [$PG_CLUSTER]"
				echo "[$(date)] [$(basename "$0")] PG_CLUSTER = [$PG_CLUSTER]" >&8
			fi
			
			if [ -z "$PG_CLUSTER" ] && [ $TO_ASK_DEL_POSTGRES == "true" ]; then
				echo "[$(date)] [$(basename "$0")] Postgres cluster is not found..." >&9

				if [ ${#RTVAR_CMDEXEC_FORCE_POSTGRES_CLUSTER_CREATION} -gt 0 ] && [ ${RTVAR_CMDEXEC_FORCE_POSTGRES_CLUSTER_CREATION} == "true" ]
				then
					echo "[$(date)] [$(basename "$0")] CMDEXEC_FORCE_POSTGRES_CLUSTER_CREATION = [$RTVAR_CMDEXEC_FORCE_POSTGRES_CLUSTER_CREATION] - Proceed to create Postgres cluster..."
					echo "[$(date)] [$(basename "$0")] CMDEXEC_FORCE_POSTGRES_CLUSTER_CREATION = [$RTVAR_CMDEXEC_FORCE_POSTGRES_CLUSTER_CREATION] - Proceed to create Postgres cluster..." >&8

					PG_CLUSTER="${AIOPS_INSTALL_NAME}-edb-postgres"
					break
				else
					# This script is not catered for peculiar situation where the Postgres cluster does not exist and is not deleted by the script!
					exit 1
				fi
			else
				echo "[$(date)] [$(basename "$0")] Postgres Cluster [$PG_CLUSTER] has been located..."
				echo "[$(date)] [$(basename "$0")] Postgres Cluster [$PG_CLUSTER] has been located..." >&8

				PG_CLUSTER_CREATION_TIME=$($OC_CMD get cluster -n $NAMESPACE $PG_CLUSTER -o jsonpath='{.metadata.creationTimestamp}')

				echo "[$(date)] [$(basename "$0")] Current Postgres cluster was created at [$PG_CLUSTER_CREATION_TIME]..."
				echo "[$(date)] [$(basename "$0")] Current Postgres cluster was created at [$PG_CLUSTER_CREATION_TIME]..." >&8

				TOPO_INVENTORY_INSTANCE=$($OC_CMD get deployment aiops-topology-inventory -n $NAMESPACE --no-headers | $AWK '{ print $2 }')

				if [ $MG_DEBUG == "true" ]
				then
					echo "[$(date)] [$(basename "$0")] TOPO_INVENTORY_INSTANCE = [$TOPO_INVENTORY_INSTANCE]"
					echo "[$(date)] [$(basename "$0")] TOPO_INVENTORY_INSTANCE = [$TOPO_INVENTORY_INSTANCE]" >&8
				fi

				if [ ${#TOPO_INVENTORY_INSTANCE} -gt 0 ]
				then
			 		TOPO_INVENTORY_INSTANCE_AVAIL=$(echo "$TOPO_INVENTORY_INSTANCE" | $AWK -F/ '{ print $1 }')
			 		TOPO_INVENTORY_INSTANCE_DESIRED=$(echo "$TOPO_INVENTORY_INSTANCE" | $AWK -F/ '{ print $2 }')

					if [ ${#TOPO_INVENTORY_INSTANCE_DESIRED} -gt 0 ]
					then
						if [ ${TOPO_INVENTORY_INSTANCE_DESIRED} -eq 0 ]
						then
							echo "[$(date)] [$(basename "$0")] Scaling up deployment [aiops-topology-inventory] to 1..."
							echo "[$(date)] [$(basename "$0")] Scaling up deployment [aiops-topology-inventory] to 1..." >&8
							$OC_CMD scale deployment aiops-topology-inventory -n $NAMESPACE --replicas=1
						fi
					else
						echo "[$(date)] [$(basename "$0")] Unable to determine TOPO_INVENTORY_INSTANCE_DESIRED! Skipping..."
						echo "[$(date)] [$(basename "$0")] Unable to determine TOPO_INVENTORY_INSTANCE_DESIRED! Skipping..." >&8
					fi
				fi

				echo "[$(date)] [$(basename "$0")] Deleting Postgres cluster..."
				echo "[$(date)] [$(basename "$0")] Deleting Postgres cluster..." >&8
				$OC_CMD delete cluster $PG_CLUSTER -n $NAMESPACE

				if [ ${#PG_CLUSTER_CREATION_TIME} -gt 0 ]
				then
					while true; do
						if isClusterDeleted "$PG_CLUSTER_CREATION_TIME"
						then
							echo "[$(date)] [$(basename "$0")] Postgres Cluster is deleted..."
							echo "[$(date)] [$(basename "$0")] Postgres Cluster is deleted..." >&8

							PG_CLUSTER_IS_DELETED="true"
							break
						else	
							echo "[$(date)] [$(basename "$0")] Postgres Cluster is still there! Sleeping for 20s..." 
							echo "[$(date)] [$(basename "$0")] Postgres Cluster is still there! Sleeping for 20s..." >&8
							sleep 20
						fi
					done
				fi

				if [ ${#PG_CLUSTER_IS_DELETED} -gt 0 ] && [ ${PG_CLUSTER_IS_DELETED} == "true" ]
				then
					break
				fi
			fi

			TO_ASK_DEL_POSTGRES="false"
			;;
		[Nn])
			# code to execute if user does not want to delete the Postgres Cluster
			echo " You have choosen not to delete the postgres cluster..."
			echo " You have choosen not to delete the postgres cluster..." >&8
			echo " Exiting..."
			echo " Exiting..." >&8
			exit 0
			;;
		*)
			echo "Invalid input. Please enter Y or N."
			echo "Invalid input. Please enter Y or N." >&8
			;;
	esac
done

echo "[$(date)] [$(basename "$0")] Restarting ODLM operator..."
echo "[$(date)] [$(basename "$0")] Restarting ODLM operator..." >&8
ODLM=$($OC_CMD get pods | grep operand | $AWK '{print $1}')

if [ -n "$ODLM" ]; then
	$OC_CMD delete pods "$ODLM" -n $NAMESPACE
	sleep 10
else
	echo "[$(date)] [$(basename "$0")] No ODLM operator found!"
	echo "[$(date)] [$(basename "$0")] No ODLM operator found!" >&8
fi

CRD_WAIT_CTR=0
CRD_WAIT_TIMEOUT=50
CRD_WAIT_SLEEP=30
    
while true; do
	(( CRD_WAIT_CTR+=1 ))

	if [ $CRD_WAIT_CTR -gt $CRD_WAIT_TIMEOUT ]
	then
		echo "[$(date)] [$(basename "$0")] [FATAL] Postgres CRD is not created! Exiting..."
		exit 1	
	fi

	CHECK_CRD=$($OC_CMD get cluster -n $NAMESPACE | grep "${AIOPS_INSTALL_NAME}-edb-postgres" | $AWK '{print $1}')
	
	if [ -n "$CHECK_CRD" ]; then
		if check_postgres_status
		then
			sleep 5
			break
		else
			echo "[$(date)] [$(basename "$0")] Fatal exception! Exiting..."
			exit 1
		fi
	else
		echo "[$(date)] [$(basename "$0")] [$CRD_WAIT_CTR] Postgres CRD not created yet! Checking..."
		echo "[$(date)] [$(basename "$0")] [$CRD_WAIT_CTR] Postgres CRD not created yet! Checking..." >&8
		sleep $CRD_WAIT_SLEEP
	fi
done

AIOPS_USERCFG=$($OC_CMD get jobs -n $NAMESPACE | grep ir-core-usercfg | $AWK '{print $1}')

if [ -n "$AIOPS_USERCFG" ]; then
	echo "[$(date)] [$(basename "$0")] Deleting job [$AIOPS_USERCFG]..."
	echo "[$(date)] [$(basename "$0")] Deleting job [$AIOPS_USERCFG]..." >&8
	$OC_CMD delete jobs -n $NAMESPACE $AIOPS_USERCFG
fi

JOBS_TO_DELETE=("aiops-ir-core-ncodl-presetup" "aiops-ir-core-usercfg-presetup" "aiops-insights-pg-setup" "aimanager-aio-create-postgres-database")

for job in "${JOBS_TO_DELETE[@]}"; do
	echo "[$(date)] [$(basename "$0")] Deleting job [$job]..."
	echo "[$(date)] [$(basename "$0")] Deleting job [$job]..." >&8
	$OC_CMD delete job "$job" -n $NAMESPACE
done

IR_OPERTATOR=$($OC_CMD get pods -n $NAMESPACE | grep ir-core-operator | $AWK '{print $1}')
COMMONUI_OPERATOR=$($OC_CMD get pods -n $NAMESPACE | grep commonui-operator | $AWK '{print $1}')
AI_OPERATOR=$($OC_CMD get pods -n $NAMESPACE | grep aimanager-operator-controller-manager | $AWK '{print $1}')
AIO_CONTROLLER=$($OC_CMD get pods -n $NAMESPACE | grep -i aimanager-aio-controller | $AWK '{print $1}')

PODS_TO_DELETE=("$IR_OPERTATOR" "$COMMONUI_OPERATOR" "$AI_OPERATOR" "$AIO_CONTROLLER")

for pod in "${PODS_TO_DELETE[@]}"; do
	echo "[$(date)] [$(basename "$0")] Deleting pod [$pod]..."
	echo "[$(date)] [$(basename "$0")] Deleting pod [$pod]..." >&8
	$OC_CMD delete pods "$pod" -n $NAMESPACE
done

drop_flyway_schema_history
sleep 10

TOPO_INV_JOB_OLD_AGE=$($OC_CMD get job aiops-topology-inventory-schema -n $NAMESPACE -o jsonpath='{.status.startTime}')

if [ $MG_DEBUG == "true" ]
then
	echo "[$(date)] [$(basename "$0")] TOPO_INV_JOB_OLD_AGE = [$TOPO_INV_JOB_OLD_AGE]"
	echo "[$(date)] [$(basename "$0")] TOPO_INV_JOB_OLD_AGE = [$TOPO_INV_JOB_OLD_AGE]" >&8
fi

echo "[$(date)] [$(basename "$0")] Deleting job [aiops-topology-inventory-schema]..."
echo "[$(date)] [$(basename "$0")] Deleting job [aiops-topology-inventory-schema]..." >&8
$OC_CMD delete jobs aiops-topology-inventory-schema -n $NAMESPACE

if [ ${#TOPO_INV_JOB_OLD_AGE} -gt 0 ] 
then
	DELETE_WAIT_CTR=0
	DELETE_WAIT_TIMEOUT=30
	DELETE_WAIT_SLEEP=5
	    
	while true; do
		(( DELETE_WAIT_CTR+=1 ))
	
		if [ $DELETE_WAIT_CTR -gt $DELETE_WAIT_TIMEOUT ]
		then
			echo "[$(date)] [$(basename "$0")] [ERROR] Timeout waiting for deletion/re-creation of job [aiops-topology-inventory-schema]! Exiting..."
			echo "[$(date)] [$(basename "$0")] [ERROR] Timeout waiting for deletion/re-creation of job [aiops-topology-inventory-schema]! Exiting..." >&8
			break
		fi

		TOPO_INV_JOB_NEW_AGE=$($OC_CMD get job aiops-topology-inventory-schema -n $NAMESPACE -o jsonpath='{.status.startTime}')

		if [ $MG_DEBUG == "true" ]
		then
			echo "[$(date)] [$(basename "$0")] TOPO_INV_JOB_NEW_AGE = [$TOPO_INV_JOB_NEW_AGE]"
			echo "[$(date)] [$(basename "$0")] TOPO_INV_JOB_NEW_AGE = [$TOPO_INV_JOB_NEW_AGE]" >&8
		fi

		if [ ${#TOPO_INV_JOB_NEW_AGE} -gt 0 ]
		then
			if [ "${TOPO_INV_JOB_NEW_AGE}" == "${TOPO_INV_JOB_OLD_AGE}" ]
			then
				echo "[$(date)] [$(basename "$0")] Still waiting for job [aiops-topology-inventory-schema] to complete deletion..."	
				echo "[$(date)] [$(basename "$0")] Still waiting for job [aiops-topology-inventory-schema] to complete deletion..." >&8
	
				sleep $DELETE_WAIT_SLEEP
			else
				if [ $MG_DEBUG == "true" ]
				then
					echo "[$(date)] [$(basename "$0")] TOPO_INV_JOB_NEW_AGE = [$TOPO_INV_JOB_NEW_AGE]"
				fi
	
				if [ ${#TOPO_INV_JOB_NEW_AGE} -eq 0 ]
				then
					echo "[$(date)] [$(basename "$0")] Unable to determine start time of the job [aiops-topology-inventory-schema]! Retrying..."
					echo "[$(date)] [$(basename "$0")] Unable to determine start time of the job [aiops-topology-inventory-schema]! Retrying..." >&8
	
					sleep $DELETE_WAIT_SLEEP
				else
					# New job created
					echo "[$(date)] [$(basename "$0")] [1] Job [aiops-topology-inventory-schema] deleted! Proceeding.."
					echo "[$(date)] [$(basename "$0")] [1] Job [aiops-topology-inventory-schema] deleted! Proceeding.." >&8
					break
				fi
			fi
		else
			# echo "[$(date)] [$(basename "$0")] Still waiting for job [aiops-topology-inventory-schema] to be re-created..."
			# echo "[$(date)] [$(basename "$0")] Still waiting for job [aiops-topology-inventory-schema] to be re-created..." >&8

			# sleep $DELETE_WAIT_SLEEP

			echo "[$(date)] [$(basename "$0")] [2] Job [aiops-topology-inventory-schema] deleted! Proceeding.."
			echo "[$(date)] [$(basename "$0")] [2] Job [aiops-topology-inventory-schema] deleted! Proceeding.." >&8
			break
		fi
	done
else
	if [ $MG_DEBUG == "true" ]
	then
		echo "[$(date)] [$(basename "$0")] [ERROR] Unable to determine start time of the job [aiops-topology-inventory-schema]! Skipping..."	
		echo "[$(date)] [$(basename "$0")] [ERROR] Unable to determine start time of the job [aiops-topology-inventory-schema]! Skipping..." >&8
	fi
fi

ASM_OPERTATOR=$($OC_CMD get pods -n $NAMESPACE | grep asm-operator | $AWK '{print $1}')
PODS_TO_DELETE=("$ASM_OPERTATOR")

for pod in "${PODS_TO_DELETE[@]}"; do
	echo "[$(date)] [$(basename "$0")] Deleting pod [$pod]..."
	echo "[$(date)] [$(basename "$0")] Deleting pod [$pod]..." >&8
	$OC_CMD delete pods "$pod" -n $NAMESPACE
done

# TODO: Check if job recreated and completed
JOB_WAIT_CTR=0
JOB_WAIT_TIMEOUT=60
JOB_WAIT_SLEEP=15
    
while true; do
	(( JOB_WAIT_CTR+=1 ))

	if [ $JOB_WAIT_CTR -gt $JOB_WAIT_TIMEOUT ]
	then
		echo "[$(date)] [$(basename "$0")] [ERROR] Job [aiops-topology-inventory-schema] still has not completed! Skipping..."
		echo "[$(date)] [$(basename "$0")] [ERROR] Job [aiops-topology-inventory-schema] still has not completed! Skipping..." >&8
		break
	fi

	TOPO_INV_JOB_STATUS=$($OC_CMD get job aiops-topology-inventory-schema -n $NAMESPACE --no-headers | $AWK '{ print $2 }')
	TOPO_INV_JOB_COMPLETE_COUNT=$(echo "$TOPO_INV_JOB_STATUS" | $AWK -F/ '{ print $1 }')
	TOPO_INV_JOB_STARTED_COUNT=$(echo "$TOPO_INV_JOB_STATUS" | $AWK -F/ '{ print $2 }')

	if [ $MG_DEBUG == "true" ]
	then
		echo "[$(date)] [$(basename "$0")] TOPO_INV_JOB_STATUS = [$TOPO_INV_JOB_STATUS] / TOPO_INV_JOB_COMPLETE_COUNT = [$TOPO_INV_JOB_COMPLETE_COUNT] / TOPO_INV_JOB_STARTED_COUNT = [$TOPO_INV_JOB_STARTED_COUNT]"
		echo "[$(date)] [$(basename "$0")] TOPO_INV_JOB_STATUS = [$TOPO_INV_JOB_STATUS] / TOPO_INV_JOB_COMPLETE_COUNT = [$TOPO_INV_JOB_COMPLETE_COUNT] / TOPO_INV_JOB_STARTED_COUNT = [$TOPO_INV_JOB_STARTED_COUNT]" >&8
	fi

	if [ ${#TOPO_INV_JOB_STATUS} -gt 0 ]
	then
		if [ ${#TOPO_INV_JOB_COMPLETE_COUNT} -gt 0 ] && [ ${#TOPO_INV_JOB_STARTED_COUNT} -gt 0 ]
		then
			if [ ${TOPO_INV_JOB_COMPLETE_COUNT} -eq ${TOPO_INV_JOB_STARTED_COUNT} ]
			then
				echo "[$(date)] [$(basename "$0")] Job [aiops-topology-inventory-schema] completed! Proceeding..."
				echo "[$(date)] [$(basename "$0")] Job [aiops-topology-inventory-schema] completed! Proceeding..." >&8
				break
			else
				echo "[$(date)] [$(basename "$0")] Waiting for job [aiops-topology-inventory-schema] to complete execution..."		
				echo "[$(date)] [$(basename "$0")] Waiting for job [aiops-topology-inventory-schema] to complete execution..." >&8
	
				sleep $JOB_WAIT_SLEEP
			fi
		else
			echo "[$(date)] [$(basename "$0")] Waiting for job [aiops-topology-inventory-schema] to complete execution..."		
			echo "[$(date)] [$(basename "$0")] Waiting for job [aiops-topology-inventory-schema] to complete execution..." >&8
	
			sleep $JOB_WAIT_SLEEP
		fi
	else
		echo "[$(date)] [$(basename "$0")] Unable to determine status of job [aiops-topology-inventory-schema]! Sleeping..."
		echo "[$(date)] [$(basename "$0")] Unable to determine status of job [aiops-topology-inventory-schema]! Sleeping..." >&8

		sleep $JOB_WAIT_SLEEP
	fi
done

echo "[$(date)] [$(basename "$0")] Scaling deployment [aiops-topology-inventory] back to 1..."
echo "[$(date)] [$(basename "$0")] Scaling deployment [aiops-topology-inventory] back to 1..." >&8
$OC_CMD scale deployment aiops-topology-inventory -n $NAMESPACE --replicas=1

